@@ -10,7 +10,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      :  Data.StateVar
--- Copyright   :  (c) Edward Kmett 2014-2015, Sven Panne 2009-2016
+-- Copyright   :  (c) Edward Kmett 2014-2015, Sven Panne 2009-2018
 -- License     :  BSD3
 -- 
 -- Maintainer  :  Sven Panne <svenpanne@gmail.com>
@@ -83,6 +83,9 @@ import Data.IORef
 import Data.Typeable
 import Foreign.Ptr
 import Foreign.Storable
+#if MIN_VERSION_base(4,12,0)
+import Data.Functor.Contravariant
+#endif
 
 --------------------------------------------------------------------
 -- * StateVar
@@ -114,6 +117,12 @@ import Foreign.Storable
 -- permit a very limited range of value assignments, and do not report failure.
 data StateVar a = StateVar (IO a) (a -> IO ()) deriving Typeable
 
+#if MIN_VERSION_base(4,12,0)
+instance Contravariant SettableStateVar where
+  contramap f (SettableStateVar k) = SettableStateVar (k . f)
+  {-# INLINE contramap #-}
+#endif
+
 -- | Construct a 'StateVar' from two IO actions, one for reading and one for
 --- writing.
 makeStateVar
@@ -227,7 +236,7 @@ instance Storable a => HasUpdate (Ptr a) a a where
 
 instance HasUpdate (IORef a) a a where
   r $~ f  = liftIO $ atomicModifyIORef r $ \a -> (f a,())
-#if __GLASGOW_HASKELL__ >= 706
+#if MIN_VERSION_base(4,6,0)
   r $~! f = liftIO $ atomicModifyIORef' r $ \a -> (f a,())
 #else
   r $~! f = liftIO $ do
